/********************************************************************************
** Form generated from reading UI file 'window.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WINDOW_H
#define UI_WINDOW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableView>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Window
{
public:
    QToolBar *toolbar;
    QWidget *centralwidget;
    QVBoxLayout *verticalLayout;
    QTabWidget *toolBox;
    QTableView *upstreamTable;
    QTableView *wlrootsTable;
    QTableView *kdeTable;
    QTableView *cosmicTable;
    QTableView *westonTable;
    QTableView *chromeosTable;
    QTableView *deepinTable;
    QTableView *frogTable;
    QTableView *gamescopeTable;
    QTableView *phoshTable;
    QTableView *unknownTable;
    QLabel *filterMessage;

    void setupUi(QMainWindow *Window)
    {
        if (Window->objectName().isEmpty())
            Window->setObjectName("Window");
        Window->resize(850, 600);
        toolbar = new QToolBar(Window);
        toolbar->setObjectName("toolbar");
        toolbar->setMovable(false);
        toolbar->setAllowedAreas(Qt::BottomToolBarArea|Qt::TopToolBarArea);
        toolbar->setFloatable(false);
        Window->addToolBar(Qt::ToolBarArea::TopToolBarArea, toolbar);
        centralwidget = new QWidget(Window);
        centralwidget->setObjectName("centralwidget");
        verticalLayout = new QVBoxLayout(centralwidget);
        verticalLayout->setSpacing(8);
        verticalLayout->setObjectName("verticalLayout");
        verticalLayout->setContentsMargins(0, 8, 0, 0);
        toolBox = new QTabWidget(centralwidget);
        toolBox->setObjectName("toolBox");
        toolBox->setDocumentMode(true);
        upstreamTable = new QTableView();
        upstreamTable->setObjectName("upstreamTable");
        toolBox->addTab(upstreamTable, QString());
        wlrootsTable = new QTableView();
        wlrootsTable->setObjectName("wlrootsTable");
        toolBox->addTab(wlrootsTable, QString());
        kdeTable = new QTableView();
        kdeTable->setObjectName("kdeTable");
        toolBox->addTab(kdeTable, QString());
        cosmicTable = new QTableView();
        cosmicTable->setObjectName("cosmicTable");
        toolBox->addTab(cosmicTable, QString());
        westonTable = new QTableView();
        westonTable->setObjectName("westonTable");
        toolBox->addTab(westonTable, QString());
        chromeosTable = new QTableView();
        chromeosTable->setObjectName("chromeosTable");
        toolBox->addTab(chromeosTable, QString());
        deepinTable = new QTableView();
        deepinTable->setObjectName("deepinTable");
        toolBox->addTab(deepinTable, QString());
        frogTable = new QTableView();
        frogTable->setObjectName("frogTable");
        toolBox->addTab(frogTable, QString());
        gamescopeTable = new QTableView();
        gamescopeTable->setObjectName("gamescopeTable");
        toolBox->addTab(gamescopeTable, QString());
        phoshTable = new QTableView();
        phoshTable->setObjectName("phoshTable");
        toolBox->addTab(phoshTable, QString());
        unknownTable = new QTableView();
        unknownTable->setObjectName("unknownTable");
        toolBox->addTab(unknownTable, QString());

        verticalLayout->addWidget(toolBox);

        filterMessage = new QLabel(centralwidget);
        filterMessage->setObjectName("filterMessage");
        QFont font;
        font.setPointSize(14);
        filterMessage->setFont(font);
        filterMessage->setAlignment(Qt::AlignCenter);

        verticalLayout->addWidget(filterMessage);

        Window->setCentralWidget(centralwidget);

        retranslateUi(Window);

        toolBox->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Window);
    } // setupUi

    void retranslateUi(QMainWindow *Window)
    {
        Window->setWindowTitle(QCoreApplication::translate("Window", "Waycheck", nullptr));
        toolbar->setWindowTitle(QCoreApplication::translate("Window", "Toolbar", nullptr));
        toolBox->setTabText(toolBox->indexOf(upstreamTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(wlrootsTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(kdeTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(cosmicTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(westonTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(chromeosTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(deepinTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(frogTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(gamescopeTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(phoshTable), QCoreApplication::translate("Window", "Page", nullptr));
        toolBox->setTabText(toolBox->indexOf(unknownTable), QCoreApplication::translate("Window", "Page", nullptr));
        filterMessage->setText(QCoreApplication::translate("Window", "No protocols match the current search", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Window: public Ui_Window {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WINDOW_H
